#include "EVChartView.h"
#include <QFontMetrics>
#include <QTime>

EVChartView::EVChartView(QQuickItem *parent):
    QQuickPaintedItem(parent),_painter(nullptr)
{
    setRenderTarget(QQuickPaintedItem::FramebufferObject);
    setAcceptHoverEvents(true);
    setAcceptedMouseButtons(Qt::NoButton);
    setFlag(ItemAcceptsInputMethod, true);
}

EVChartView::~EVChartView()
{
}

void EVChartView::setLine(QStringList list)
{
    //选择时间后 读取数据库后传入
    if(list.size()<0) return;
    _vLine.clear();
    double chartWmin=m_leftDisplayWidth;
    double chartWmax=_width-m_rightDisplayWidth;
    double chartHmin=m_headerHeight;
    double chartHmax=_height-m_bottomHeight;

    //反转坐标系 坐标为（0,1）==>转换为(chartWMin+0,chartHMax-1)
    double xRate =(double)(chartWmax-chartWmin)/(m_xMax-m_xMin);

    //y轴长度
    int chartHeightLength = chartHmax-chartHmin;
    _xMinTmp=m_xMin;
    _xMaxTmp=m_xMax;
    _yMinTmp=m_yMin;
    _yMaxTmp=m_yMax;
    double yRate =((double)chartHeightLength)/(m_yMax-m_yMin);
    _xSpace =xRate;
    _ySpace=yRate;

    if(list.size()>0){
        for(int index =0; index<list.size();index++){
            if(index==0){
                continue;
            }
            double xStartP=0.f,yStartP=0.f;
            double xEndP=0.f,yEndP=0.f;
            xStartP=(double)chartWmin+(xRate*((double)index-1-m_xMin));
            double ystart= list[index-1].toDouble();
            //y匹配坐标系
            yStartP=m_headerHeight+yRate*(m_yMax- ystart);
            xEndP =(double)chartWmin+xRate*((double)index-m_xMin);
            yEndP =(double)m_headerHeight+yRate*(m_yMax-list[index].toDouble());
            QLine line =QLine(xStartP,yStartP,xEndP,yEndP);
            _vLine.append(line);
        }
    }

    update();
}

void EVChartView::clearLine()
{
    _vLine.clear();
    _vSeeAreaLine.clear();
    update();
}

void EVChartView::startTest()
{
    m_xMin = 0;
    m_xMax = 1000;
    m_yMin = 0;
    m_yMax = 100;

    QStringList list;

    for (int i = 0; i < m_xMax; ++i){
        int num = qrand() % 100;

        list.append(QString::number(num));
    }

    setLine(list);
}

void EVChartView::paint(QPainter *painter)
{
    _painter = painter;
    setFont();
    _width =width();
    _height=height();
    _chartFrame=QRectF(m_leftDisplayWidth,m_headerHeight,_width-m_leftDisplayWidth
                       -m_rightDisplayWidth,_height-m_headerHeight-m_bottomHeight);

    drawHeader();
    drawFrame();
    drawHorizontalLine();
    drawLeftTickLabel();
    drawBottomTitle();
    drawVlines();
    drawLeftLabel();

    _painter->setClipRect(_chartFrame);
}

void EVChartView::hoverEnterEvent(QHoverEvent *)
{

}

void EVChartView::hoverLeaveEvent(QHoverEvent *)
{

}

void EVChartView::hoverMoveEvent(QHoverEvent *)
{

}

void EVChartView::wheelEvent(QWheelEvent *)
{

}

void EVChartView::drawHeader()
{
    if(m_headerHeight>0 &&!m_headerTxt.isEmpty()){
        int widthOfTitle =_painter->fontMetrics().horizontalAdvance(m_headerTxt);//.width(m_headerTxt);//字符串显示的像素大小
        int heightOffset = _painter->fontMetrics().height();

        if(m_headerTxtColor != "")
            _painter->setPen(QColor(m_headerTxtColor));  //设置画笔颜色

        _painter->drawText(this->width()/2-widthOfTitle/2,m_headerHeight/2+heightOffset/2,m_headerTxt);
    }
}

void EVChartView::drawFrame()
{
    QVector<QLine> vLine;
    //top
    QLine topLine =QLine(m_leftDisplayWidth,m_headerHeight,_width-m_rightDisplayWidth,m_headerHeight);
    vLine.append(topLine);
    QLine bottomLine =QLine(m_leftDisplayWidth,_height-m_bottomHeight,_width-m_rightDisplayWidth,_height-m_bottomHeight);
    vLine.append(bottomLine);
    QLine leftLine =QLine(m_leftDisplayWidth,m_headerHeight,m_leftDisplayWidth,_height-m_bottomHeight);
    vLine.append(leftLine);
    QLine rightLine =QLine(_width-m_rightDisplayWidth,m_headerHeight,_width-m_rightDisplayWidth,_height-m_bottomHeight);
    vLine.append(rightLine);
    _painter->setPen(QColor("black"));
    _painter->drawLines(vLine);
}

void EVChartView::drawHorizontalLine()
{
    _yTickCountTmp = m_vTickCount;
    QStringList list = stepZoom(_yMinTmp,_yMaxTmp);

    _yTickCountTmp =0;
    if(list.size()==0) return;
    QVector<QLine> vLine;
    double space =(_height-m_headerHeight-m_bottomHeight)/(list.size()-1);
    for(int index =0; index <list.size()-1;index++){
        QLine line =QLine(m_leftDisplayWidth,index*space+m_headerHeight, _width-m_rightDisplayWidth,index*space+m_headerHeight);
        vLine.append(line);
    }
    _painter->drawLines(vLine);
}

void EVChartView::drawLeftTickLabel()
{
    _yTickCountTmp =m_vTickCount;
    QStringList list = stepZoom(_yMinTmp,_yMaxTmp);
    _yTickCountTmp =0;
    //算出xlabel的位置
    //y 间距
    double displayYHeight =double(_height-m_headerHeight-m_bottomHeight);
    double space =displayYHeight/(list.size()-1);
    //x坐标
    //    double yLabelPoint =m_leftDisplayWidth;
    //匹配y轴长度和单位
    //    double yLen = ((double)_yMaxTmp- (double)_yMinTmp)/list.size();
    for(int index =0; index <list.size();index++){
        int heightOffset = _painter->fontMetrics().height();
        //字体右对齐 多出负号 或出现位数不一样
        QRectF point(0,index*space+m_headerHeight-heightOffset/2,\
                     m_leftDisplayWidth-10,index*space+m_headerHeight);
        _painter->drawText(point,Qt::AlignRight,list.at(list.size()-1-index));
    }
}

void EVChartView::drawBottomTitle()
{
    //算出xlabel的位置
    _yTickCountTmp =m_hTickCount;
    QStringList list = stepZoom(_xMinTmp,_xMaxTmp);

    _yTickCountTmp =0;
    //x 间距
    double displayXWidth =double(_width-m_rightDisplayWidth- m_leftDisplayWidth);
    double space =displayXWidth/(list.size()-1);
    //y 坐标
    double yLabelPoint =_height-m_bottomHeight;
    //匹配x轴长度和单位
    double xLen = ((double)_xMaxTmp- (double)_xMinTmp)/m_hTickCount;
    for(int nSpaceIndex =0; nSpaceIndex<list.size();nSpaceIndex++){
        int widthOfTitle =_painter->fontMetrics().horizontalAdvance(QString::number(_xMinTmp+xLen*nSpaceIndex));//字符串显示的像素大小
        int heightOffset = _painter->fontMetrics().height();
        _painter->drawText(m_leftDisplayWidth+space*nSpaceIndex-widthOfTitle/2,yLabelPoint+heightOffset,\
                           QString::number((int)list.at(nSpaceIndex).toDouble()) );
    }
}

void EVChartView::drawLeftLabel()
{
    _painter->translate(m_leftDisplayWidth/2-4,_height/2);
    _painter->rotate(270);// 旋转
    QFont font;
    font.setPointSize(11);
    font.setFamily("Microsoft YaHei");
    font.setLetterSpacing(QFont::AbsoluteSpacing,1);
    _painter->setFont(font);
    _painter->setPen("black");
    _painter->drawText( 0,0,m_leftLabel);
    //    _painter->rotate(-90);
    _painter->translate(_width/2,_height/2);
    setFont();
}

void EVChartView::drawVlines()
{
    if(_vLine.size()>0){
        _painter->setPen("blue");
        //将左边的和右边的去掉
        if(!_isScale){
            _painter->drawLines(_vLine);
        }

        else{
            getSeeAreaLine();
            _painter->drawLines(_vSeeAreaLine);
        }
    }
}

QStringList EVChartView::getCurrentFrame(const QString &strframeValue)
{
    return strframeValue.split("&");
}

void EVChartView::pointScale(const QPoint &point)
{
    double xMinTmp =_xMinTmp,xMaxTmp =_xMaxTmp,yMaxTmp =_yMaxTmp,yMinTmp =_yMinTmp;
    double xWidth =(_width-m_leftDisplayWidth-m_rightDisplayWidth);
    double yHeight =_height-m_headerHeight-m_bottomHeight;
    //放大后的长宽
    double xCurrentScaleWidth  =xWidth*getXScaleValue();
    double yCurrentScaleHeight =yHeight*getYScaleValue();
    //x y位置
    double x =point.x()-m_leftDisplayWidth;
    double y=point.y()-m_headerHeight;
    //x y 与width height比例
    double xRate = x/xWidth;
    double yRate = y/yHeight;
    //宽 左右两边长度,高  上下长度
    double xLeftWidth    = xCurrentScaleWidth*xRate;
    double xRightWidth   = xCurrentScaleWidth*((double)1-xRate);
    double yTopHeight    = yCurrentScaleHeight*yRate;
    double yBottomHeight = yCurrentScaleHeight*((double)1-yRate);
    //宽坐标  0-24000
    double xPointLen =_xMaxTmp-_xMinTmp; double yPointLen =_yMaxTmp-_yMinTmp;
    _xMinTmp = (_xMinTmp+ (xLeftWidth-x)*(xPointLen/xCurrentScaleWidth));
    _xMaxTmp = _xMaxTmp-(xRightWidth-(xWidth-x))*(xPointLen/xCurrentScaleWidth);
    //高坐标 -8-8
    //    _yMaxTmp = m_yMax- (yTopHeight-y)*(yPointLen/yCurrentScaleHeight);
    //    _yMinTmp = m_yMin+(yBottomHeight-(yHeight-y))*(yPointLen/yCurrentScaleHeight);
    _yMaxTmp  =_yMaxTmp -(yTopHeight-y)*(yPointLen/yCurrentScaleHeight);
    _yMinTmp  =_yMinTmp +(yBottomHeight-(yHeight-y))*(yPointLen/yCurrentScaleHeight);;


    if((_xMaxTmp-_xMinTmp)/m_hTickCount<5 ||(_yMaxTmp-_yMinTmp)/m_vTickCount<0.002){
        _xMinTmp=xMinTmp;
        _xMaxTmp=xMaxTmp;
        _yMinTmp=yMinTmp;
        _yMaxTmp=yMaxTmp;
    }

    _isScale=true;


}

void EVChartView::pointNarrow(const QPoint &point)
{
    //缩小后刻度
    qDebug()<<"缩小前 xMin："<<_xMinTmp<<"   缩小前 xMax："<<_xMaxTmp;
    double xWidth =(_width-m_leftDisplayWidth-m_rightDisplayWidth);
    double yHeight =_height-m_headerHeight-m_bottomHeight;
    //放大后的长宽
    double xCurrentScaleWidth  =xWidth/getXScaleValue();
    double yCurrentScaleHeight =yHeight/getYScaleValue();
    //x y位置
    double x =point.x()-m_leftDisplayWidth;
    double y=point.y()-m_headerHeight;
    //x y 与width height比例  缩小前比例
    double xRate =(_xMaxTmp-_xMinTmp)/xCurrentScaleWidth;
    double yRate = (_yMaxTmp-_yMinTmp)/yCurrentScaleHeight;

    //放大后x 坐标位置  = 左边扩展的坐标+缩小后的坐标位置
    double xNarrowAddWidth =xWidth-xCurrentScaleWidth;  //增加的width
    //左边加了多少
    double xLeft = (x/xWidth) *xNarrowAddWidth;
    _xMinTmp=_xMinTmp -xRate*xLeft ;


    //右边
    double xRight = xNarrowAddWidth- (x/xWidth) *xNarrowAddWidth;
    _xMaxTmp=_xMaxTmp +xRate*xRight ;
    //缩小后 y坐标的位置 =上扩展坐标+缩小后的坐标位置
    double yNarrowAddWidth =yHeight-yCurrentScaleHeight;  //增加的width
    double yTop = (y/yHeight) *yNarrowAddWidth;
    _yMaxTmp =_yMaxTmp+yTop*yRate;
    double yBottom =yNarrowAddWidth- (y/yHeight) *yNarrowAddWidth;
    _yMinTmp =_yMinTmp-yBottom*yRate;
    qDebug()<<"缩小后 xMin："<<_xMinTmp<<"   缩小后 xMax："<<_xMaxTmp;
    _isScale=true;
}

void EVChartView::getSeeAreaLine()
{
    //获取可视区域
    _vSeeAreaLine.clear();
    QVector<QLine> tmp ;
    double chartWmin=m_leftDisplayWidth;
    double chartWmax=_width-m_rightDisplayWidth;
    double chartHmin=m_headerHeight;
    double chartHmax=_height-m_bottomHeight;
    double xRate =(double)(chartWmax-chartWmin)/(_xMaxTmp-_xMinTmp);

    //y轴长度
    int chartHeightLength = chartHmax-chartHmin;
    double yRate =((double)chartHeightLength)/(_yMaxTmp-_yMinTmp);
    //选择时间后 读取数据库后传入
    QStringList list= getCurrentFrame("");
    if(list.size()>0){
        int index =_xMinTmp;
        int max =_xMaxTmp;
        int tmp11 =0;
        for( ; index<max;index++){
            if(index>=list.size()){
                break;
            }
            if(index<=0){
                //x 将值算好
                tmp11++;
                continue;
            }
            tmp11++;
            double xStartP=0.f,yStartP=0.f;
            double xEndP=0.f,yEndP=0.f;
            xStartP=(double)chartWmin+(xRate*((double)tmp11-1/*-_xMinTmp*/));
            double ystart= list[index-1].toDouble();
            //y匹配坐标系
            yStartP=m_headerHeight+yRate*(_yMaxTmp- ystart);
            xEndP =(double)chartWmin+xRate*((double)tmp11/*-_xMinTmp*/);
            yEndP =(double)m_headerHeight+yRate*(_yMaxTmp-list[index].toDouble());
            QLine line =QLine(xStartP,yStartP,xEndP,yEndP);
            tmp.append(line);
            _vSeeAreaLine.append(line);
        }
    }
}

double EVChartView::initDouble(double &bRate)
{
    QString num=QString::number(bRate,'f',8);
    //取到不为0的开始 补齐为5   假如为0.0004 则补齐为 0.0005 每次加
    return 0.0;
}

double EVChartView::getYScaleValue()
{
    _yScaleCount++;
    if(_yScaleCount<=0) return   1;
    return _yScale;
}

double EVChartView::getXScaleValue()
{
    _xScaleCount++;
    if(_xScaleCount<=0) return   1;

    return _xScale;
}

void EVChartView::setFont()
{
    QFont font;
    font.setPointSize(10);
    font.setFamily("Microsoft YaHei");
    font.setLetterSpacing(QFont::AbsoluteSpacing,0);
    _painter->setFont(font);
    _painter->setRenderHint(QPainter::Antialiasing);
}

QString EVChartView::removeZero(double data)
{
    QString label;
    QRegExp reg;
    reg.setPattern("(\\.){0,1}0+$");
    QString dds = QString("%1").arg(data, 0, 'f', 2).replace(reg, "");
    label=dds;
    return  "";
}

bool EVChartView::isXaxisInt()
{
    double xLen = ((double)_xMaxTmp- (double)_xMinTmp)/m_hTickCount;
    if(isInt(xLen)){
        return true;
    }else{
        //偏移 正数  如果大于0.5 向前取整 小于0.5向后取整
        int data =xLen;
        double leftData =xLen-data;
        if(leftData>0.5){
            xLen =data+1;
        }else{
            xLen =data;
        }
    }
    //将 轴计算出来
    _xMaxTmp =_xMinTmp+xLen*m_hTickCount;

    return true;
}

bool EVChartView::isInt(const double &dData)
{
    int nData = (int)dData;
    if(nData==dData){
        return  true;
    }else{
        return  false;
    }
}

QStringList EVChartView::stepZoom(double &min, double &max)
{
    //最小步径
    QStringList list;
    list<<"0"<<"1"<<"2"<<"4"<<"5"<<"6"<<"7"<<"8";
    if(min==0 &&max ==0) return list;
    list.clear();
    double  dlen = max-min;
    int  nlen =0;
    bool isSacleData =false;
    if(dlen<=100){
        nlen = dlen*10000;
        isSacleData =true;
    }else{
        nlen =dlen;
    }

    int  step=((nlen))%(5*_yTickCountTmp);
    if(step==0){
        step =((nlen))/(5*_yTickCountTmp);
        for(int index=0 ;index<_yTickCountTmp+1;index++){
            if(isSacleData){
                list<<QString::number(min+(double)(step)*index*5/10000,'f',3);
            }else{
                list<<QString::number(min+(double)(step)*index*5,'f',3);
            }
        }
        return list;
    }

    step=((nlen))%(2*_yTickCountTmp);
    if(step==0){
        step =((nlen))/(2*_yTickCountTmp);
        for(int index=0 ;index<_yTickCountTmp+1;index++){

            if(isSacleData){
                list<<QString::number(min+(double)(step)*index*2/10000,'f',3);
            }else{
                list<<QString::number(min+(double)(step)*index*2,'f',3);
            }
        }
        return list;
    }
    step=((nlen))%(_yTickCountTmp);
    if(step==0){
        step =((nlen))/(_yTickCountTmp);
        for(int index=0 ;index<_yTickCountTmp+1;index++){
            if(isSacleData){
                list<<QString::number(min+(double)(step)*index/10000,'f',3);
            }else{
                list<<QString::number(min+(double)(step)*index,'f',3);
            }
        }
        return list;
    }
    //未返回 补长度 将长度补为5  2  1中最短的
    int step5=((nlen))%(5*_yTickCountTmp);
    int step2 =((nlen))%(2*_yTickCountTmp);
    int step1 =((nlen))%(1*_yTickCountTmp);
    int value =0;
    //判断1 2 5 并补齐
    if(step5>step2){
        if(step5>step1){
            value =step5;
        }else{
            value =step1;
        }
    }else{
        if(step2>step1){
            value =step2;
        }else{
            value =step1;
        }
    }
    if(value==step5){
        //补回去补齐
        double maxTmp =0.f;
        double minTmp =0.f;
        if(isSacleData){
            maxTmp  = max+  ((double)((5*_yTickCountTmp)) -step5)/10000/2;
            minTmp  = min-  ((double)((5*_yTickCountTmp) -step5))/10000/2;
            nlen = (maxTmp-minTmp)*10000;
        }else{
            maxTmp  = max+  ((double)((_yTickCountTmp)) -step5)/2;
            minTmp  = min-  ((double)((_yTickCountTmp) -step5))/2;
            nlen = (maxTmp-minTmp);
        }
        step=nlen/(5*_yTickCountTmp);
        for(int index=0 ;index<(_yTickCountTmp+1);index++){
            if(isSacleData){
                list<<QString::number(minTmp+(double)(step)*index/10000,'f',3);
            }else{
                list<<QString::number(minTmp+(double)(step)*index,'f',3);
            }
        }
        min =minTmp;
        max =maxTmp;
        return  list;
    }else if(value==step2){

        double maxTmp =0.f;
        double minTmp =0.f;
        if(isSacleData){
            maxTmp  = max+  ((double)((_yTickCountTmp)) -step2)/10000/2;
            minTmp  = min-  ((double)((_yTickCountTmp) -step2))/10000/2;
            nlen = (maxTmp-minTmp)*10000;
        }else{
            maxTmp  = max+  ((double)((_yTickCountTmp)) -step2)/2;
            minTmp  = min-  ((double)((_yTickCountTmp) -step2))/2;
            nlen = (maxTmp-minTmp);
        }
        step=nlen/(2*_yTickCountTmp);
        for(int index=0 ;index<(_yTickCountTmp+1);index++){
            if(isSacleData){
                list<<QString::number(minTmp+(double)(step)*index/10000,'f',3);
            }else{
                list<<QString::number(minTmp+(double)(step)*index,'f',3);
            }
        }
        min =minTmp;
        max =maxTmp;
        return  list;
    }else{
        double maxTmp =0.f;
        double minTmp =0.f;
        if(isSacleData){
            maxTmp  = max+  ((double)((_yTickCountTmp)) -step1)/10000/2;
            minTmp  = min-  ((double)((_yTickCountTmp) -step1))/10000/2;
            nlen = (maxTmp-minTmp)*10000;
        }else{
            maxTmp  = max+  ((double)((_yTickCountTmp)) -step1)/2;
            minTmp  = min-  ((double)((_yTickCountTmp) -step1))/2;
            nlen = (maxTmp-minTmp);
        }
        step=nlen/(_yTickCountTmp);
        for(int index=0 ;index<(_yTickCountTmp+1);index++){
            if(isSacleData){
                list<<QString::number(minTmp+(double)(step)*index/10000,'f',3);
            }else{
                list<<QString::number(minTmp+(double)(step)*index,'f',3);
            }
        }
        min =minTmp;
        max =maxTmp;
        return  list;
    }
}

