#ifndef EVChartView_H
#define EVChartView_H

#include <QObject>
#include <QQuickPaintedItem>
#include "PropertyHelper.h"
#include <QPainter>
#include <QList>
#include <QMutex>


class EVChartView : public QQuickPaintedItem
{
    Q_OBJECT
public:
    EVChartView(QQuickItem *parent = 0);
    ~EVChartView();

    AUTO_PROPERTY(int,vTickCount,5)
    AUTO_PROPERTY(int,hTickCount,5)
    AUTO_PROPERTY(QString,headerTxt,"")
    AUTO_PROPERTY(QString,headerTxtColor,"")
    AUTO_PROPERTY(int,headerHeight,20)
    AUTO_PROPERTY(int,leftDisplayWidth,0)
    AUTO_PROPERTY(int,rightDisplayWidth,0)
    AUTO_PROPERTY(int,bottomHeight,0)
    AUTO_PROPERTY(int,xMax,24000)
    AUTO_PROPERTY(int,xMin,-24000)
    AUTO_PROPERTY(int,yMax,10)
    AUTO_PROPERTY(int,yMin,0)
    AUTO_PROPERTY(QString,leftLabel,"")
public:
    Q_INVOKABLE void setLine(QStringList list);
    Q_INVOKABLE void clearLine();


    Q_INVOKABLE void startTest();    //测试数据  支持百万级瞬绘

protected:
    void paint(QPainter *painter) override;
protected:
    //--------------鼠标事件-----------------------------
    //鼠标进入事件
    virtual void hoverEnterEvent(QHoverEvent *event) override;
    //鼠标离开事件
    virtual void hoverLeaveEvent(QHoverEvent *event) override;
    //鼠标移动事件
    virtual void hoverMoveEvent(QHoverEvent *event)  override;
    //鼠标滚轮事件
    virtual void wheelEvent(QWheelEvent *event) override;

private:
    //----------------------chartView paint操作----------------------------------
    void drawHeader();                                          //画Header
    void drawFrame();                                           //画外边框
    void drawHorizontalLine();                                  //画水平线

    void drawLeftTickLabel();                                   //画垂直线
    //void drawLeftTitle();                                       //画垂直的单位
    void drawBottomTitle();                                     //画底部单位
    void drawLeftLabel();                                       //

private:
    //------------------- 画线 放大 缩小 操作---------------------------------------------
    void drawVlines();                                          //画线 Qt自动将重合线去重
    QStringList getCurrentFrame(const QString & strframeValue); //获取当前可画线区域
    void pointScale(const QPoint &point);                       //放大
    void pointNarrow(const QPoint &point);                      //缩小
    void getSeeAreaLine();                                      //生成折线
    double initDouble(double &bRate);                           //将第一次比例初始化
    double getYScaleValue();                                    //获取y放大倍数
    double getXScaleValue();                                    //获取x放大倍数
    void  setFont();
    //一格小于100操作

private:
    QString removeZero(double data);
    bool    isXaxisInt();
    bool    isInt(const double &dData);
    QStringList  stepZoom(double &min,double &max);

private:
    QPainter*       _painter;
    double          _width=0.f;
    double          _height =0.f;
    QVector<QLine>  _vLine;
    QRectF          _chartFrame;
    QPoint          _currentPos;
    double          _xSpace=0;             //基本x比例
    double          _ySpace=0;             //基本y比例
    double          _xCurrentSpace=0;      //当前x比例
    double          _yCurrentSpace=0;      //当前y比例
    double          _xMinTmp=0;
    double          _xMaxTmp=0;
    double          _yMinTmp=0;
    double          _yMaxTmp=0;
    int             _scaleCount =0;
    int             _scaleXCount =0;
    QVector<QLine>  _vSeeAreaLine;

    double          _xScale =2;        //x放大倍数
    double          _yScale =2;          //y放大倍数
    int             _xScaleCount =0;     //x放大次数
    int             _yScaleCount =0;     //y放大次数
    bool            _isScale  = false;
    int             _yTickCountTmp =0;
    int             _xTickCountTmp =0;

};

#endif // EVChartView_H
