#include "Dashboard.h"

#include <qmath.h>

const  int radius = 180;                //半径
const  int maxv = 300;                  //最大刻度值
const  int minv = 0;                    //最小刻度值

const int longstep = 10;                //长刻度步长
const int shortstep = maxv/longstep;    //短刻度步长

const double startDraw = -135.0;        //绘制起始角度
const double stopDraw = 270.0;          //绘制结束角度

Dashboard::Dashboard(QQuickItem *parent):
    QQuickPaintedItem(parent)
{
    Values = 0;
}

Dashboard::~Dashboard()
{

}

void Dashboard::setValues(unsigned int &iValues)
{
    Values = iValues;
    update();
}

void Dashboard::paint(QPainter *painter)
{
    painter->setRenderHint(QPainter::Antialiasing);
    QPoint center(width()/2,height()/2);
    painter->save();

    //绘制背景
    painter->translate(center);
    painter->setPen(Qt::NoPen);
    painter->setBrush(QColor(142,142,142)); // 设置背景颜色
    int bkRadius = radius+3;
    painter->drawEllipse(QPoint(),bkRadius ,bkRadius);

    //绘制刻度线
    painter->rotate(startDraw);
    for (int i = 0; i <=shortstep; i ++) {
        painter->setPen(i<shortstep*0.8?QColor(32,243,32):QColor(243,32,32));
        if(i%(shortstep/longstep) != 0)
        {
            QPoint p1(0,-(radius - 8));
            QPoint p2(0,-radius);
            painter->drawLine(p1,p2);
        }else{
            QPoint p1(0,-(radius - 14));
            QPoint p2(0,-radius);
            painter->drawLine(p1,p2);
        }
        painter->rotate(stopDraw/shortstep);
    }

    painter->restore();
    QFontMetrics fm = painter->fontMetrics();
    int tx = center.x();
    int ty =int(center.y()+(radius/1.4));

    //title
    QString title = "仪表盘";
    QSize tsz = fm.size(0,title);
    QRect trect(QPoint(tx-tsz.width()/2,ty-tsz.height()/2),tsz);
    painter->setPen(QColor(127,127,127));
    painter->setBrush(QColor(173,163,163));
    painter->drawRoundedRect(trect.adjusted(-6,-4,6,4),3,3);
    painter->setPen(QColor(243,243,243));
    painter->setBrush(Qt::NoBrush);
    painter->drawText(trect,title);

    painter->setPen(QColor(243,243,243));
    int hand = radius -21;
    for (int i = 0;i<= longstep ;i++) {
        qreal angle = qDegreesToRadians(startDraw+i*(stopDraw/longstep));
        QString etext = QString::number(minv+i*(maxv - minv)/longstep);
        QPointF dirvec(qSin(angle),-qCos(angle));
        QPointF ecenter(center.x()+hand*dirvec.x(),center.y()+hand*dirvec.y());
        QSize esz = fm.size(0,etext);
        QRectF erect(QPointF(ecenter.x()-esz.width()/2,ecenter.y()-esz.height()/2),esz);
        painter->drawText(erect,etext);
    }

    //仪表盘指针绘制
    painter->save();
    QPoint triangle[]={
        {-5,0},
        {0,30-radius},
        {5,0}
    };

    qreal degree = startDraw + stopDraw*(Values - minv)/(maxv - minv);
    painter->translate(center);
    painter->rotate(degree);
    painter->setPen(Qt::NoPen);
    painter->setBrush(Qt::red);
    painter->drawPolygon(triangle,3);
    painter->setPen(QPen(QColor(214,0,0),2));
    painter->drawEllipse(QPoint(),7,7);
    painter->restore();
}
